"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// A grid is a data structure which stores and retrive points with a 2d
// coordinate system. The nodes are stored in a sparse 2d array.
var ArrayGrid = exports.ArrayGrid = function () {
  function ArrayGrid() {
    _classCallCheck(this, ArrayGrid);

    // A row major, sparse 2d array used to hold nodes in a 2d grid.
    this.data = [
      // [], [],
    ];

    this.maxCols = 0;

    this.positions = new Map();
  }

  // Create an instance from a 2D, row major, uniform column sized array.


  _createClass(ArrayGrid, [{
    key: "get",


    // If x or y are invalid, returns undefined.
    value: function get(x, y) {
      if (y < 0 || y >= this.data.length) {
        // out of bounds
        return;
      }

      var row = this.data[y];
      if (x < 0 || x >= row.length) {
        // out of bounds
        return;
      }

      return row[x];
    }
  }, {
    key: "put",
    value: function put(thing, x, y) {
      if (x < 0 || y < 0) {
        throw Error("Index out of bounds");
      }

      if (this.data.length <= y) {
        // not long enough, grow data
        this.data.length = y + 1;
      }

      if (!this.data[y]) {
        // need to create array
        this.data[y] = [];
      }

      var row = this.data[y];
      if (row.length <= x) {
        row.length = x + 1;

        if (this.maxCols < row.length) {
          this.maxCols = row.length;
        }
      }

      this.positions.set(thing, { x: x, y: y });
      row[y] = thing;
    }

    // The number of rows this grid has.

  }, {
    key: "rows",
    value: function rows() {
      return this.data.length;
    }

    // The number of columns this grid has.

  }, {
    key: "columns",
    value: function columns() {
      return this.maxCols;
    }
  }, {
    key: "positionOf",
    value: function positionOf(item) {
      return this.positions.get(item);
    }
  }, {
    key: "shallowCopy",
    value: function shallowCopy() {
      // copy data
      var newThing = Object.assign({}, this);

      // copy behavior
      var proto = Object.getPrototypeOf(this);
      Object.setPrototypeOf(newThing, proto);

      return newThing;
    }
  }], [{
    key: "fromArray",
    value: function fromArray(rows) {
      var grid = new ArrayGrid();

      grid.data = rows;
      grid.maxCols = rows.reduce(function (acc, val) {
        if (val.length > acc) {
          return val.length;
        }

        return acc;
      }, 0);

      grid.data.forEach(function (row, rowIndex) {
        return row.forEach(function (item, colIndex) {
          grid.positions.set(item, { x: colIndex, y: rowIndex });
        });
      });

      return grid;
    }

    // Creates a grid array from existing data.

  }, {
    key: "fromData",
    value: function fromData(_ref) {
      var data = _ref.data,
          maxCols = _ref.maxCols,
          positions = _ref.positions;

      var grid = new ArrayGrid();
      return Object.assign(grid, { data: data, maxCols: maxCols, positions: positions });
    }
  }]);

  return ArrayGrid;
}();