'use strict';

var _ = require('./');

var _states = require('./puzzles/states');

var grid = void 0;

beforeEach(function () {
  var nodeRows = _states.rows.map(function (row) {
    return row.split("").map(function (khar) {
      return new _.CharNode(khar);
    });
  });
  expect(nodeRows).toHaveLength(_states.rows.length);

  grid = _.ArrayGrid.fromArray(nodeRows);
});

it('should be the correct shape', function () {
  expect(grid.rows()).toBe(_states.rows.length);
  expect(grid.columns()).toBe(_states.rows[0].length);
});

it('should have its origin at the top left', function () {
  // check four corners
  expect(grid.get(0, 0).value).toBe('T');
  expect(grid.get(0, 32).value).toBe('V');
  expect(grid.get(32, 0).value).toBe('S');
  expect(grid.get(32, 32).value).toBe('Z');
});

it('should allow fast lookups', function () {
  var node = grid.get(10, 10);
  expect(grid.positionOf(node)).toMatchObject({ x: 10, y: 10 });
});

it('should shallow copy', function () {
  var copiedGrid = grid.shallowCopy();

  expect(copiedGrid).not.toBe(grid);
  expect(copiedGrid.data).toBe(grid.data);
  expect(copiedGrid.columns()).toBe(grid.columns());
});