"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findMatches = findMatches;
// A brute force algorithm for finding the solutions of a wordsearch. The worst
// case complexity of this algorithm is O(n * w * l) where n is the number of
// nodes in the graph, w is the number of words we are searching for and l is
// the average length of the word we are searching for.
function findMatches(words, grid, directions) {
  var matches = {
    // word: [
    //     [nodes],
    //     [nodes],
    // ],
  };

  var cols = grid.columns();
  var rows = grid.rows();

  // travarse graph once for each word
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = words[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var word = _step.value;

      for (var x = 0; x < cols; x++) {
        for (var y = 0; y < rows; y++) {
          var node = grid.get(x, y);
          if (!node) {
            // no node here
            continue;
          }

          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = directions[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var _ref = _step2.value;
              var _ref$displacement = _ref.displacement;
              var dx = _ref$displacement.x;
              var dy = _ref$displacement.y;

              // look at every direction of every node

              var discoveredWord = [];

              for (var index = 0, cx = x, cy = y; index < word.length; index++, cx += dx, cy += dy) {
                var currentNode = grid.get(cx, cy);

                // check whether this node continues the sequence
                if (!currentNode || currentNode.value !== word[index]) {
                  // match failed, try next
                  break;
                }

                // it matches, add to discovered
                discoveredWord.push(currentNode);
              }

              // we can be here in three possible states:
              // * the word length is zero
              // * there wasn't a match
              // * a complete match was found

              if (word.length <= 0 || word.length !== discoveredWord.length) {
                // the word length is zero or there wasn't a match
                continue;
              }

              // a complete match was found, record match
              if (!matches[word]) {
                matches[word] = [];
              }

              var discoveredWordArray = [discoveredWord];
              matches[word] = matches[word].concat(discoveredWordArray);

              // add to every node in the word for easy access
              var _iteratorNormalCompletion3 = true;
              var _didIteratorError3 = false;
              var _iteratorError3 = undefined;

              try {
                for (var _iterator3 = discoveredWord[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                  var _node = _step3.value;

                  _node.partOf = _node.partOf.concat(discoveredWordArray);
                }
              } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion3 && _iterator3.return) {
                    _iterator3.return();
                  }
                } finally {
                  if (_didIteratorError3) {
                    throw _iteratorError3;
                  }
                }
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return matches;
}