'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.findMatches = findMatches;
function findMatches(words, grid, directions) {
  var matches = {
    // word: [
    //     [nodes],
    //     [nodes],
    // ],
  };

  var cols = grid.columns();
  var rows = grid.rows();

  var prefixLength = Math.min(words.map(function (d) {
    return d.length;
  }).reduce(function (a, b) {
    return Math.min(a, b);
  }, Infinity), 5);

  var prefixTable = new Map();

  // build prefix table
  for (var x = 0; x < cols; x++) {
    for (var y = 0; y < rows; y++) {
      var node = grid.get(x, y);
      if (!node) {
        // no node here
        continue;
      }

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = directions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _ref = _step.value;
          var _ref$displacement = _ref.displacement;
          var dx = _ref$displacement.x;
          var dy = _ref$displacement.y;

          // look at every direction of every node

          var prefixNodes = [];

          for (var index = 0, cx = x, cy = y; index < prefixLength; index++, cx += dx, cy += dy) {
            var currentNode = grid.get(cx, cy);

            // check whether this node continues the sequence
            if (!currentNode) {
              // match failed, try next
              break;
            }

            // it matches, add to discovered
            prefixNodes.push(currentNode);
          }

          // we can be here in two possible states:
          // * hit an empty spot and the prefix isn't complete
          // * a complete match was found

          if (prefixNodes.length < prefixLength) {
            // incomplete prefix
            continue;
          }

          var prefix = prefixNodes.map(function (node) {
            return node.value;
          }).join('');

          // a complete prefix was found, record it
          if (!prefixTable.has(prefix)) {
            prefixTable.set(prefix, []);
          }

          var prefixTableEntries = prefixTable.get(prefix);
          prefixTableEntries.push(prefixNodes);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = words[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var word = _step2.value;

      var prefixKey = word.substring(0, prefixLength);
      var prefixes = prefixTable.get(prefixKey);

      // If the word shorter than the prefix, no match is found.

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = prefixes[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _prefix = _step3.value;

          var _prefix2 = _slicedToArray(_prefix, 2),
              firstNode = _prefix2[0],
              secondNode = _prefix2[1];

          var _grid$positionOf = grid.positionOf(firstNode),
              fx = _grid$positionOf.x,
              fy = _grid$positionOf.y;

          var _grid$positionOf2 = grid.positionOf(secondNode),
              sx = _grid$positionOf2.x,
              sy = _grid$positionOf2.y;

          var _dx = sx - fx;
          var _dy = sy - fy;

          var possibleMatch = [];

          for (var _index = 0, _cx = fx, _cy = fy; _index < word.length; _index++, _cx += _dx, _cy += _dy) {
            var _currentNode = grid.get(_cx, _cy);

            // check whether this node continues the sequence
            if (!_currentNode || _currentNode.value !== word[_index]) {
              // match failed, try next
              break;
            }

            possibleMatch.push(_currentNode);
          }

          // we can be here in two possible states:
          // * an incomplete match was found
          // * a complete match was found

          if (word.length <= 0 || word.length !== possibleMatch.length) {
            continue;
          }

          // record match
          if (!matches[word]) {
            matches[word] = [];
          }

          var discoveredWordArray = [possibleMatch];
          matches[word] = matches[word].concat(possibleMatch);

          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = possibleMatch[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var _node = _step4.value;

              _node.partOf = _node.partOf.concat(discoveredWordArray);
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return matches;
}